//
//  RALatchTriggerTest.m
//  AudioHijackKit2
//
//  Created by Michael Ash on 11/7/08.
//  Copyright 2008 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "RALatchTriggerTest.h"

#import "RALatchTrigger.h"


@implementation RALatchTriggerTest

- (void)testBasics
{
	RALatchTrigger *trigger = [[RALatchTrigger alloc] init];
	STAssertNotNil( trigger, @"Couldn't allocate trigger" );
	
	// test multiple signals, no assert to be done here because a failure just blocks
	int i;
	for( i = 0; i < 100; i++ )
		[trigger signal];
	
	NSThread *thread = [[NSThread alloc] initWithTarget: self selector: @selector( _basicsThread: ) object: trigger];
	[thread start];
	
	NSTimeInterval start = [NSDate timeIntervalSinceReferenceDate];
	while( [NSDate timeIntervalSinceReferenceDate] - start < 1.0 && !_sharedFlag )
		usleep( 1000 );
	
	STAssertTrue( _sharedFlag, @"thread failed to set shared flag after signaling" );
	
	_sharedFlag = NO;
	usleep( 10000 );
	STAssertFalse( _sharedFlag, @"thread altered shared flag with no signal!" );
	
	[trigger signal];
	
	start = [NSDate timeIntervalSinceReferenceDate];
	while( [NSDate timeIntervalSinceReferenceDate] - start < 1.0 && !_sharedFlag )
		usleep( 1000 );
	
	STAssertTrue( _sharedFlag, @"thread failed to set shared flag after signaling" );
	
	[thread cancel];
	[thread release];
	[trigger release];
}

- (void)_basicsThread: (RALatchTrigger *)trigger
{
	NSThread *thread = [NSThread currentThread];
	while( ![thread isCancelled] )
	{
		NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
		
		[trigger wait];
		_sharedFlag = YES;
		
		[pool release];
	}
}

@end
